// scene.js

"use strict";

/*global THREE: true,
         window: true,
         document: true,
         requestAnimationFrame: true,
         scene: true
*/

var keyQ1 = new THREE.Quaternion();



var axisX = new THREE.Vector3(1, 0, 0);
var axisY = new THREE.Vector3(0, 1, 0);
var axisZ = new THREE.Vector3(0, 0, 1);

// 
function rotateOnAxis(object, axis, angle) {
    keyQ1.setFromAxisAngle(axis, angle);
    //object.quaternion.multiply(keyQ1);
    object.quaternion.copy(keyQ1);
}


function dumbbell(shereRadius, cylinderRadius, colour) {
    var geometry,
        material,
        mesh,
        group;

    group = new THREE.Object3D();      // Create container

    geometry = new THREE.SphereGeometry(shereRadius, 32, 32);
    material = new THREE.MeshPhongMaterial({color: colour});
    mesh = new THREE.Mesh(geometry, material);
    mesh.position.y = 1.0;
    group.add(mesh);

    mesh = new THREE.Mesh(geometry, material);
    mesh.position.y = -1.0;
    group.add(mesh);

    geometry = new THREE.CylinderGeometry(cylinderRadius, cylinderRadius, 2, 32);
    mesh = new THREE.Mesh(geometry, material);
    group.add(mesh);

    return group;
}

function showAxes() {
    var xPointer,
        yPointer,
        zPointer;

    // The color coding is X-Y-Z => red-green-blue. 
    // X axis pointer 
    xPointer = dumbbell(0.1, 0.04, 0xff0000);
    rotateOnAxis(xPointer, axisZ, Math.PI / 2);
    scene.add(xPointer);

    // Y axis pointer 
    yPointer = dumbbell(0.1, 0.04, 0x00ff00, axisX, 0);
    rotateOnAxis(yPointer, axisY, Math.PI / 2);
    scene.add(yPointer);

    // Z axis pointer
    zPointer = dumbbell(0.1, 0.04, 0x0000ff, axisX, 0);
    rotateOnAxis(zPointer, axisX, Math.PI / 2);
    scene.add(zPointer);
}

// Create a scene.
var scene = new THREE.Scene();
var aspectRatio = window.innerWidth / window.innerHeight;
var width = window.innerWidth;
var height = window.innerHeight;
var dirLight;
var hemiLight;

// Create a camera
var camera = new THREE.PerspectiveCamera(30, aspectRatio, 0.1, 1000);
camera.position.x = 2.0;
camera.position.y = -2.0;
camera.position.z = 2.0;
camera.up = new THREE.Vector3(0, 0, 1);
camera.lookAt(new THREE.Vector3(0.2, 0.2, 0.2));

// Create lights

hemiLight = new THREE.HemisphereLight(0xffffff, 0xffffff, 0.6);
hemiLight.color.setHSL(0.6, 1, 0.6);
hemiLight.groundColor.setHSL(0.095, 1, 0.75);
hemiLight.position.set(0, 100, 0);
scene.add(hemiLight);

//

dirLight = new THREE.DirectionalLight(0xffffff, 1);
dirLight.color.setHSL(0.1, 1, 0.95);
scene.add(dirLight);

showAxes();

// Add a cube to the scene
var material = new THREE.MeshPhongMaterial({color: 0xffff00,
                                        specular: 0xa0a0a0,
                                        shininess: 15,
                                        vertexColors: THREE.FaceColors,
                                        shading: THREE.FlatShading
                                       });
var geometry = new THREE.BoxGeometry(0.6, 2, 0.1);
var cube = new THREE.Mesh(geometry, material);
cube.position.x = 0.5;
cube.position.y = 0.5;
cube.position.z = 0.5;
scene.add(cube);


geometry = new THREE.TorusGeometry(1, 0.02, 16, 100);
var material = new THREE.MeshPhongMaterial({color: 0x0000ff,
                                        specular: 0xa0a0a0,
                                        shininess: 4,
                                        vertexColors: THREE.FaceColors,
                                        shading: THREE.FlatShading
                                       });
var torus = new THREE.Mesh(geometry, material);
scene.add(torus);

geometry = new THREE.TorusGeometry(1, 0.02, 16, 100);
var material = new THREE.MeshPhongMaterial({color: 0x00ff00,
                                        specular: 0xa0a0a0,
                                        shininess: 4,
                                        vertexColors: THREE.FaceColors,
                                        shading: THREE.FlatShading
                                       });
torus = new THREE.Mesh(geometry, material);
rotateOnAxis(torus, axisX, Math.PI / 2);
scene.add(torus);

geometry = new THREE.TorusGeometry(1, 0.02, 16, 100);
var material = new THREE.MeshPhongMaterial({color: 0xff0000,
                                        specular: 0xa0a0a0,
                                        shininess: 4,
                                        vertexColors: THREE.FaceColors,
                                        shading: THREE.FlatShading
                                       });
torus = new THREE.Mesh(geometry, material);
rotateOnAxis(torus, axisY, Math.PI / 2);
scene.add(torus);





// Create a renderer
var renderer = new THREE.WebGLRenderer();
renderer.setSize(width, height);
document.body.appendChild(renderer.domElement);

// Render function.
var render = function () {
    requestAnimationFrame(render);

    //rotateOnAxis(cube, axisX, 0.01); 
    //rotateOnAxis(cube, axisY, 0.01); 

    renderer.render(scene, camera);
};

render();

function onWindowResize() {
    camera.aspect = window.innerWidth / window.innerHeight;
    camera.updateProjectionMatrix();
    renderer.setSize(window.innerWidth, window.innerHeight);
}


function onKeyDown(event) {
    switch (event.keyCode) {
    case 72: // h
        break;
    case 68: // d
        break;
    case 38: // Up 
        rotateOnAxis(cube, axisY, +0.9);
        break;
    case 40: // Down 
        rotateOnAxis(cube, axisY, -0.9);
        break;
    case 37: // Left
        rotateOnAxis(cube, axisZ, +0.9);
        break;
    case 39: // Right
        rotateOnAxis(cube, axisZ, -0.9);
        break;
    }
}

window.addEventListener('resize', onWindowResize, false);
document.addEventListener('keydown', onKeyDown, false);



function random(low, high) {
    return Math.random() * (high - low) + low;
}

// Test
console.log(q0, q1, q2, q3);

var gx = 0.0;
var gy = 0.0;
var gz = 0.0;
var ax = 1.0;
var ay = 1.0;
var az = 1.0;
var mx = 0.0;
var my = 0.0;
var mz = 0.0;



var samplePeriodMillis = 1000 / sampleFreq;
var sampleTime = 0;

function doTest() {

    // Gyro input
    gx =   0.0;
    gy =   0.0;
    gz =   0.0;

    // Accelerometer input
    ax = 0.2 * Math.cos(5 * sampleTime / sampleFreq);       // Roll oscillations
    ay = 0.2 * Math.sin(5 * sampleTime / sampleFreq);       // Pitch oscillations
    az = 1.0;                                    // Gravity constant

    // Magnetometer input
    mx =  Math.cos(sampleTime / sampleFreq);         // Rotate around the verticle axis (z)
    my =  Math.sin(sampleTime / sampleFreq);
    mz =  0.0;

    // Move the light around with the magnetometer vector. 
    dirLight.position.set(10 * Math.sin(sampleTime / sampleFreq),
                          10 * Math.cos(sampleTime / sampleFreq),
                          5);

    // Put some noise on the gyro
    var noiseLevel = 0.05;
    var noise = function () { return random(-noiseLevel,  +noiseLevel); };
    gx += noise();
    gy += noise();
    gz += noise();
    ax += noise();
    ay += noise();
    az += noise();
    mx += noise();
    my += noise();
    mz += noise();

    madgwickAHRSupdate(gx, gy, gz, ax, ay, az, mx, my, mz);

    // Note: Seems to me Madgwick has put the quaternian axis vector in q1, q2, q3 and the angle in q0
    //       
    cube.quaternion.set(q1, q2, q3, q0);

    if ((q0 === NaN) || (q1 === NaN) || (q2 === NaN) || (q3 === NaN)) {
    	console.log ("Exploded!");
    }

    sampleTime += samplePeriodMillis;
}

setInterval(function () {
    doTest();
}, samplePeriodMillis);
